@@ -242,16 +242,22 @@ QTextStream& formatCode(QTextStream &s, const QString& code, Indentor &indentor)
         }
     }
 
+    static QRegExp emptyLine("\\s*[\\r]?[\\n]?\\s*");
+
     foreach(QString line, lst) {
-        while (line.end()->isSpace())
-            line.chop(1);
-        int limit = 0;
-        for(int i = 0; i < spacesToRemove; ++i) {
-            if (!line[i].isSpace())
-                break;
-            limit++;
+        if (!line.isEmpty() && !emptyLine.exactMatch(line)) {
+            while (line.end()->isSpace())
+                line.chop(1);
+            int limit = 0;
+            for(int i = 0; i < spacesToRemove; ++i) {
+                if (!line[i].isSpace())
+                    break;
+                limit++;
+            }
+
+            s << indentor << line.remove(0, limit);
         }
-        s << indentor << line.remove(0, limit) << endl;
+        s << endl;
     }
     return s;
 }
@@ -276,6 +282,7 @@ QString Generator::translateType(const AbstractMetaType *cType,
                                  Options options) const
 {
     QString s;
+    static int constLen = strlen("const");
 
     if (context && cType &&
         context->typeEntry()->isGenericClass() &&
@@ -287,22 +294,36 @@ QString Generator::translateType(const AbstractMetaType *cType,
         s = "void";
     } else if (cType->isArray()) {
         s = translateType(cType->arrayElementType(), context, options) + "[]";
-    } else if (cType->isEnum() || cType->isFlags()) {
-        if (options & Generator::EnumAsInts)
-            s = "int";
-        else
-            s = cType->cppSignature();
+    } else if (options & Generator::EnumAsInts && (cType->isEnum() || cType->isFlags())) {
+        s = "int";
     } else {
-        s = cType->cppSignature();
-        if (cType->isConstant() && (options & Generator::ExcludeConst)) {
-            // Remove just the first ‘const’, avoiding removal of template attr.
-            int pos = s.indexOf("const");
-            s.remove(pos, 5); //remove strlen(const)
-        }
-        if (cType->isReference() && (options & Generator::ExcludeReference)) {
-            // Remove just the first ‘&’, avoiding removal of template references.
-            int pos = s.indexOf("&");
-            s.remove(pos, 1); //remove strlen(const)
+        if (options & Generator::OriginalName) {
+            s = cType->originalTypeDescription().trimmed();
+            if ((options & Generator::ExcludeReference) && s.endsWith("&"))
+                s = s.left(s.size()-1);
+
+            // remove only the last const (avoid remove template const)
+            if (options & Generator::ExcludeConst) {
+                int index = s.lastIndexOf("const");
+
+                if (index >= (s.size() - (constLen + 1))) // (VarType const)  or (VarType const[*|&])
+                    s = s.remove(index, constLen);
+            }
+        } else if (options & Generator::ExcludeConst || options & Generator::ExcludeReference) {
+            AbstractMetaType* copyType = cType->copy();
+
+            if (options & Generator::ExcludeConst)
+                copyType->setConstant(false);
+
+            if (options & Generator::ExcludeReference)
+                copyType->setReference(false);
+
+            s = copyType->cppSignature();
+            if (!copyType->typeEntry()->isCppPrimitive())
+                s.prepend("::");
+            delete copyType;
+        } else {
+            s = cType->cppSignature();
         }
     }
 
@@ -322,3 +343,29 @@ QString Generator::subDirectoryForPackage(QString packageName) const
     return QString(packageName).replace(".", QDir::separator());
 }
 
+template<typename T>
+static QString getClassTargetFullName_(const T* t, bool includePackageName)
+{
+    QString name = t->name();
+    const AbstractMetaClass* context = t->enclosingClass();
+    while (context) {
+        name.prepend('.');
+        name.prepend(context->name());
+        context = context->enclosingClass();
+    }
+    if (includePackageName) {
+        name.prepend('.');
+        name.prepend(t->package());
+    }
+    return name;
+}
+
+QString getClassTargetFullName(const AbstractMetaClass* metaClass, bool includePackageName)
+{
+    return getClassTargetFullName_(metaClass, includePackageName);
+}
+
+QString getClassTargetFullName(const AbstractMetaEnum* metaEnum, bool includePackageName)
+{
+    return getClassTargetFullName_(metaEnum, includePackageName);
+}
